######################################################################
# SUPPORTING INFORMATION -  R SNA SCRIPT FILE (SCRIPT TO RUN SNA OF THE MAIN TEXT)
#
#   These are the R scripts to run SNA present in 'Network structure and the
# optimisation of proximity-based association criteria' (Gomes et al. Methods 
# in Ecology and Evolution), organized in the following parts:
#
### MAKE NETWORK BASED ON INDIVIDUALS' CO-OCCURRENCE -- COUNTS OF CO-OCCURRENCES
### MAKE NETWORK BASED ON INDIVIDUALS' TIME OVERLAP WITH TEMPORAL AND SPATIAL OPTIMIZATION -- TIME OVERLAP 
### STRUCTURE-BASED AND EXTERNAL OPTIMISATION OF ASSOCIATION CRITERIA - DESCRIPTIVE STATISTICS CALCULATION
### NETWORK RANDOMIZATIONS: CHECK SUGNIFICANCE
#
#   Notes:
#    All codes of this script file are dependent on the upload of the 'R_SNA_data' and 'SNA_functions_code' files,
# both present in the supporting information 'SNA_files'
#    Please read the section "Structure-based and external optimisation of association criteria" in
# the methods of our article to guide you through this code.
#    R version 3.6.1
#    packages required are described in each function codes file and section
#    Created by Ana Cristina R. Gomes with input from remaining authors
#
######################################################################



# import functions to run SNA reported in the main manuscript (needs to be in the same R directory)
source("SNA_functions_code.R")

## this 'SNA_functions_code' file contains the following functions
### 'gmm_adapted', code for GMM, from 'asnipe' R package, to performe the method described in Ferreira et al. (2019, DOI: 10.1101/739789),
##### that allows the calculation of an association matrix based on individuals' co-occurrence, posteriously
### 'overlap_network_spatial_temporal', adapted from the code wrote in Ferreira et al. (2019,  DOI: 10.1101/739789), that
##### calculates an association matrix based on time overlap data between individuals, considering the information on the start and end
##### of an individuals' occurrence, and individuals within a spatial criterion and a lag time in their time of occurrence
### 'stats_calculation', calculates the descriptive statitsics (CV, SD, Entropy, and Assortment), described in the main text
### 'customed_randomization', makes permutations on the Group-by-Individual (GBI) matrix and on the data to calculate time overlaps



# load the raw cleaned data needed to run SNA reported in the main manuscript (needs to be in the same R directory)
load("R_SNA_data.RData")

## this 'R_SNA_data' file contains the following objects
#### data, is the raw cleaned data, with the following columns:
##### "Date", with date (day-month-year) information
##### "Reader", with RFID reader ID. To run only feeders or only perches analyses, data needes to be subsetted according to reader ID;
####### feeder ID's: 90.1, 90.2, 90.3, 90.4, 100.1, 100.2, 100.3, 100.4; perches ID's: 10, 20, 30, 40, 50, 60, 70, 80
##### "Tag", with individuals' ID
##### "Start", with the time, in seconds, the individual arrived to the RFID reader
##### "End", with the time, in seconds, the individual left the RFID reader
#### distance_matrixes_list, is a list of matrixes reader by reader with distances, in meters, between all the the readers;
###### names of each matrix in the list corresponding to the initial date (in seconds) of the change in the distances
#### assortment_groups, is a matrix with two columns: the first column "individual_ID" has the ID of all individuals in the data,
###### and the second column "group" has the number of the group that each individual belongs





##### MAKE NETWORK BASED ON INDIVIDUALS' CO-OCCURRENCE -- COUNTS OF CO-OCCURRENCES #####

# define gathering events based on two GMM, according to Ferreira et al. (2019,  DOI: 10.1101/739789) method

co_occurrence<-gmm_adapted(data=data)


# extract objects obtained through "gmm_adapted"
co_occurrence_gbi_final<-co_occurrence$gbi_final
co_occurrence_events_final<-co_occurrence$events_final
data_with_minutes<-co_occurrence$data_with_minutes

# create association matrix based on co-occurrency of individuals, accordimg to the gathering events recognized before,
## by the 2 GMM's
library(asnipe) # version 1.1.11
network_cooccurrence<-get_network(co_occurrence_gbi_final, data_format = "GBI", association_index = "SRI")



##### MAKE NETWORK BASED ON INDIVIDUALS' TIME OVERLAP WITH TEMPORAL AND SPATIAL OPTIMIZATION -- TIME OVERLAP #####


# the following code line allows the calculation of an association matrix with one combination of spatial and temporal criteria

overlap<- overlap_network_spatial_temporal(data=data, distance_matrixes=distance_matrixes_list, spatial_criterion=0.4, temporal_criterion=0)

# to calculate association matrixes for all the combinations of spatial and temporal criteria needed (as presented in the main text)
## it is needed to run the code line as many time as possible combinations


# extract objects obtained through "overlap_network_spatial_temporal"
overlap_Totaltime<-overlap$Totaltime
overlap_dyads<-overlap$dyads_overlap

# extract association matrix based on time overlap between individuals
network_overlap<-overlap$network_overlap_time



##### STRUCTURE-BASED AND EXTERNAL OPTIMISATION OF ASSOCIATION CRITERIA #####


# this code line calculates descriptive statistics for one combination of spatial and temporal criteria, for both counts of co-occurrence and time overlap networks
# the code line can be adapted to calculate all the combinations of spatial and temporal criteria

network_statistics<-stats_calculation(network_overlap=network_overlap, network_cooccurrence=network_cooccurrence, assortment_groups=assortment_groups)


# extract the statistics calculated, obtained through "overlap_network_spatial_temporal"
network_statistics$overlap_cv
network_statistics$cooccurrence_cv

network_statistics$overlap_sd
network_statistics$cooccurrence_sd

network_statistics$overlap_entropy
network_statistics$cooccurrence_entropy

network_statistics$overlap_assortment
network_statistics$cooccurrence_assortment



#### NETWORK RANDOMIZATIONS: CHECK SUGNIFICANCE ####

# check if the network calculated from the best association criteria is significantly more structured when compared to a null random model based on permutations
# the following code line makes a list of data and GBI matrixes that allows the calculation of both types of network, afterwards

data_randomised<-customed_randomization(data=data_with_minutes, network_cooccurrence=network_cooccurrence, gbi_final=co_occurrence_gbi_final, permutation_number = 100000, events_final=co_occurrence_events_final)


# create an object to make a list of networks from permuted data, to calculate statistics afterwards
networks_permuted<-list()

# in the main text significance was only checked for the networks resulted from the best association criteria, independetly selected for each descriptive statistic

# for descriptive statistics that select as best association criteria, the counts of co-occurrence network (as it is the case of the SD in the main text)
for(i in 1:length(data_randomised$gbi_swapped)){
  networks_permuted[[i]]<-get_network(data_randomised$gbi_swapped[[i]], data_format = "GBI", association_index = "SRI")
}

# for descriptive statistics that select as best association criteria, one from the spatial and temporal optimisation of the time overlap network (as it is the case of the CV, Entropy, and Assortment in the main text)
for(i in 1:length(data_randomised$overlap_swapped)){
  networks_permuted[[i]]<-overlap_network_spatial_temporal(data=data_randomised$overlap_swapped[[i]], distance_matrixes=distance_matrixes_list, spatial_criterion=0.4, temporal_criterion=0)$network_overlap_time
print(i)
  }


# this originates a list of networks 'networks_permuted' that can be used to calculate a specific descriptive statistic (CV, SD, Entropy, or Assortment)
### and to compare with the 'true' observed value, calculated previously from the 'stats_calculation' function, present in the 'network_statistics' object

# after calculating the distribution of descriptive statistics from the permuted data, P values can be calculated with the following code:
P<-2*sum(statistics_permuted_data>=statistic_observed_data)/(length(networks_permuted))
# for further details on this see Farine and Whitehead (2015, DOI: 10.1111/1365-2656.12418), and Farine (2017, DOI:10.1111/2041-210X.12772)
